$:.unshift File.join(Dir.pwd, 'lib')

require 'date'
require 'firestore-odm'

class Post < FirestoreODM::Model
  path :posts

  field :content, String
  field :privacy, String, :in => ['public', 'hidden', 'private']
  field :created_at, DateTime
end

class Account < FirestoreODM::Model
  path :info

  field :first_name, String
  field :last_name, String
  field :birthday, Date, :max => Proc.new { Date.today.prev_year(18) }
  field :updated_at, DateTime
end

class User < FirestoreODM::Model
  path :users

  field :name, String
  field :created_at, DateTime

  contains Post, Account
end

puts Account.parse_birthday(Date.parse('2001-12-02'))
puts Account.parse_birthday(Date.parse('2002-12-02'))
puts Account.parse_birthday(Date.parse('2003-12-02'))
puts Post.parse_privacy('public')
puts Post.parse_privacy('none')

begin
  user = User.create_with_id 'john.smith' do |doc|
    doc.name = 'John Smith'
    doc.created_at = DateTime.now
  end

  puts "#{user.path}: #{user.to_json}"

  begin
    user.create_account_with_id 'account' do |doc|
      doc.first_name = 'John'
      doc.last_name = 'Smith'
      doc.birthday = Date.parse('1981-06-25')
      doc.updated_at = DateTime.now
    end

    begin
      account = user.account_from_id 'account'

      puts "#{account.path}: #{account.to_json}"
    ensure
      account.delete
    end

    post = user.create_post do |doc|
      doc.content = 'Hello world!'
      doc.privacy = 'public'
      doc.created_at = DateTime.now
    end

    begin
      puts "#{post.path}: #{post.to_json}"
    ensure
      post.delete
    end
  ensure
    user.delete
  end
end